"use strict";


function KeyboardTestHelper()
{
   var expected;
   
   this.init = init;

   function init(isGui)
   {
      var cfg = {
         isGui : isGui
      };
      p2j.keyboard.init(cfg);
      p2j.clipboard = {
            enabled : false,
            needsSelection : function() {},
            haveSelection  : function() {},
            clearSelection : function() {}
      };
      p2j.socket = {
            send : function(data)
            {
               var ch = data[1] << 8 | data[2];
               addResult(document.testform.results, ch);
            }};
      
      document.testform.testArea.value = '';
      document.testform.results.value = '';
      document.testform.errors.value = '';
      expected = [];
   }

   this.addRange = addRange;
   
   function addRange(beg, end)
   {
      for (var i = beg; i <= end; i++)
      {
         expected[i] = 1;
      }
   }
   
   this.addRepeatsAt = addRepeatsAt;
   
   function addRepeatsAt(index, count)
   {
      expected[index] = count;
   }
   
   function addResult(el, t)
   {
      var old = el.value;
      if (old === "")
      {
         el.value = t;
      }
      else
      {
         el.value = old + '\n' + t;
      }
   }
   
   this.isPassed = isPassed;
   
   function isPassed()
   {
      var testValue = document.testform.results.value;
      var lines = testValue.split('\n');
      var actualResults = [];
      var oldValue = null;
      var repeatsCount = 0;
      for (var i = 0; i < lines.length; i++)
      {
         var newValue = parseInt(lines[i], 10);
         if (oldValue === newValue)
         {
            repeatsCount++;
         }
         else
         {
            oldValue = newValue;
            repeatsCount = 1;
         }
         actualResults[newValue] = repeatsCount;
      }
      var passed = true;
      if (expected.length !== actualResults.length)
      {
         addResult(document.testform.errors, "expected length is " + expected.length
               + ", but actual length is " + actualResults.length);
      }
      for(var i = 0; i < expected.length; i++)
      {
         if (expected[i] !== undefined)
         {
            if (actualResults[i] !== expected[i])
            {
               addResult(document.testform.errors, i + " expected key with repeats " + expected[i] +
                     ", but actual key with repeats " + actualResults[i]);
               return false;
            }
         }
      }
      addResult(document.testform.errors, "Test is passed!");
      return passed;
   }
}

p2j.test_helper = new KeyboardTestHelper();

/**
 * exception: plus ESC code 27, repeats two times
 */
function ctrlKeysGroup(altKey, shiftKey, metaKey)
{
// 0     CTRL-@
   simulateKeyPress("testArea", '2'.codePointAt(0), true, altKey, shiftKey, metaKey);
//   1     CTRL-A
   simulateKeyPress("testArea", 'A'.codePointAt(0), true, altKey, shiftKey, metaKey);
//   2     CTRL-B
   simulateKeyPress("testArea", 'B'.codePointAt(0), true, altKey, shiftKey, metaKey);
//   3     CTRL-C
   simulateKeyPress("testArea", 'C'.codePointAt(0), true, altKey, shiftKey, metaKey);
//   4     CTRL-D
   simulateKeyPress("testArea", 'D'.codePointAt(0), true, altKey, shiftKey, metaKey);
//   5     CTRL-E
   simulateKeyPress("testArea", 'E'.codePointAt(0), true, altKey, shiftKey, metaKey);
//   6     CTRL-F
   simulateKeyPress("testArea", 'F'.codePointAt(0), true, altKey, shiftKey, metaKey);
//   7     CTRL-G                           BELL
   simulateKeyPress("testArea", 'G'.codePointAt(0), true, altKey, shiftKey, metaKey);
// 10     CTRL-J
   simulateKeyPress("testArea", 'J'.codePointAt(0), true, altKey, shiftKey, metaKey);
//  11     CTRL-K
   simulateKeyPress("testArea", 'K'.codePointAt(0), true, altKey, shiftKey, metaKey);
//  12     CTRL-L
   simulateKeyPress("testArea", 'L'.codePointAt(0), true, altKey, shiftKey, metaKey);
// 14     CTRL-N
// 15     CTRL-O
// 16     CTRL-P
// 17     CTRL-Q
// 18     CTRL-R
// 19     CTRL-S
// 20     CTRL-T
// 21     CTRL-U
// 22     CTRL-V
// 23     CTRL-W
// 24     CTRL-X
// 25     CTRL-Y
// 26     CTRL-Z
   for (var code = 78; code <= 90; code++)
   {
      simulateKeyPress("testArea", code, true, altKey, shiftKey, metaKey);
   }
   if (!altKey && !shiftKey && !metaKey)
   {
    // 27     ESC                              END-ERROR
    //because of CHUI it needs to call ESC two times to clear internal esc state.
       simulateKeyPress("testArea", 27, false, false, false, false);
       simulateKeyPress("testArea", 27, false, false, false, false);
   }
// 28     CTRL-'\'
   simulateKeyPressExt("testArea", 220, 92, true, altKey, shiftKey, metaKey);
//  29     CTRL-]
   simulateKeyPressExt("testArea", 221, 93, true, altKey, shiftKey, metaKey);
//  30     CTRL-^
   simulateKeyPress("testArea", 54, true, altKey, shiftKey, metaKey);
//  31     CTRL-_
   simulateKeyPressExt("testArea", 173, 45, true, altKey, shiftKey, metaKey);
}

/**
 * exception: plus  DEL (46)   DELETE-CHARACTER Progress4GL code 127
 */
function printedKeysGroup(ctrlKey, altKey, metaKey)
{
// 32     space
   simulateKeyPress("testArea", 32, ctrlKey, altKey, false, metaKey);

   //~ 192 - 96 = 192 - 126
   //! 49 - 33
   //@ 50 - 64
   //# 51 - 35
   //$ 52 - 36
   //% 53 - 37
   //^ 54 - 94
   //& 55 - 38
   //* 56 - 42
   //( 57 - 40
   //) 48 - 41
   //_ 173 - 45 = 173 - 95
   //+ 61 - 43
   //| 220-92 = 220 - 124
   // [ 219 - 91 = 219 - 123
   // ] 221 - 93 = 221 - 125
   // ; 59 - 58
   // ' 222 - 39 = 222 - 34
   // , 188 - 44 = 188 - 60
   // . 190 - 46 = 190 - 62
   // / 191 - 47 = 191 - 63

// SHIFT + .
//   33     !                                ! = SHIFT + '1'
   simulateKeyPress("testArea", 49, ctrlKey, altKey, true, metaKey);
//   34     "                                " = SHIFT + '\''
   simulateKeyPressExt("testArea", 222, 39, ctrlKey, altKey, true, metaKey);
//   35     #                                # = SHIFT + '3'
   simulateKeyPress("testArea", 51, ctrlKey, altKey, true, metaKey);
//   36     $                                $ = SHIFT + '4'
   simulateKeyPress("testArea", 52, ctrlKey, altKey, true, metaKey);
//   37     %                                % = SHIFT + '5'
   simulateKeyPress("testArea", 53, ctrlKey, altKey, true, metaKey);
//   38     &                                & = SHIFT + '7'
   simulateKeyPress("testArea", 55, ctrlKey, altKey, true, metaKey);

// 39     '                                '
   simulateKeyPressExt("testArea", 222, 39, ctrlKey, altKey, false, metaKey);

//   40     (                                ( =  SHIFT + '('
   simulateKeyPress("testArea", 57, ctrlKey, altKey, true, metaKey);
//   41     )                                ) =  SHIFT + ')'
   simulateKeyPress("testArea", 48, ctrlKey, altKey, true, metaKey);
//   42     *                                * = SHIFT +'8'
   simulateKeyPress("testArea", 56, ctrlKey, altKey, true, metaKey);
//   43     +                                + = SHIFT + '='
   simulateKeyPress("testArea", 61, ctrlKey, altKey, true, metaKey);
//   44     ,                                ,
   simulateKeyPressExt("testArea", 188, 44, ctrlKey, altKey, false, metaKey);
//   45     -                                -
   simulateKeyPressExt("testArea", 173, 45, ctrlKey, altKey, false, metaKey);
//   46     .                                .
   simulateKeyPressExt("testArea", 190, 46, ctrlKey, altKey, false, metaKey);
//   47     /                                /
   simulateKeyPressExt("testArea", 191, 47, ctrlKey, altKey, false, metaKey);
   //digits
//   48 ... 57    0...9
   for (var code = 48; code <= 57; code++)
   {
      simulateKeyPress("testArea", code, ctrlKey, altKey, false, metaKey);
   }
//   58     :                                : = SHIFT + ';'
   simulateKeyPress("testArea", 59, ctrlKey, altKey, true, metaKey);
//   59     ;                                ;
   simulateKeyPress("testArea", 59, ctrlKey, altKey, false, metaKey);
//   60     <                                <
   simulateKeyPressExt("testArea", 188, 44, ctrlKey, altKey, true, metaKey);
//   61     =                                =
   simulateKeyPress("testArea", 61, ctrlKey, altKey, false, metaKey);
//   62     >                                >
   simulateKeyPressExt("testArea", 190, 46, ctrlKey, altKey, true, metaKey);
//   63     ?                                ?
   simulateKeyPressExt("testArea", 191, 47, ctrlKey, altKey, true, metaKey);
//   64     @                                @
   simulateKeyPress("testArea", 50, ctrlKey, altKey, true, metaKey);
//   65     A                                A
//   66     B                                B
//   67     C                                C
//   68     D                                D
//   69     E                                E
//   70     F                                F
//   71     G                                G
//   72     H                                H
//   73     I                                I
//   74     J                                J
//   75     K                                K
//   76     L                                L
//   77     M                                M
//   78     N                                N
//   79     O                                O
//   80     P                                P
//   81     Q                                Q
//   82     R                                R
//   83     S                                S
//   84     T                                T
//   85     U                                U
//   86     V                                V
//   87     W                                W
//   88     X                                X
//   89     Y                                Y
//   90     Z                                Z
   for (var code = 65; code <= 90; code++)
   {
      simulateKeyPress("testArea", code, ctrlKey, altKey, true, metaKey);
   }
//   91     [                                [
   simulateKeyPressExt("testArea", 219, 91, ctrlKey, altKey, false, metaKey);
//   92     \                                \
   simulateKeyPressExt("testArea", 220, 92, ctrlKey, altKey, false, metaKey);
//   93     ]                                ]
   simulateKeyPressExt("testArea", 221, 93, ctrlKey, altKey, false, metaKey);
//   94     ^                                ^ = SHIFT + '6'
   simulateKeyPress("testArea", 54, ctrlKey, altKey, true, metaKey);
//   95     _                                _ = SHIFT + '-'
   simulateKeyPressExt("testArea", 173, 45, ctrlKey, altKey, true, metaKey);
//   96     `                                `
   simulateKeyPressExt("testArea", 192, 96, ctrlKey, altKey, false, metaKey);

//   97     a                                a
//   98     b                                b
//   99     c                                c
//   100     d                                d
//   101     e                                e
//   102     f                                f
//   103     g                                g
//   104     h                                h
//   105     i                                i
//   106     j                                j
//   107     k                                k
//   108     l                                l
//   109     m                                m
//   110     n                                n
//   111     o                                o
//   112     p                                p
//   113     q                                q
//   114     r                                r
//   115     s                                s
//   116     t                                t
//   117     u                                u
//   118     v                                v
//   119     w                                w
//   120     x                                x
//   121     y                                y
//   122     z                                z
   if (!altKey)
   {
      for (var code = 65; code <= 90; code++)
      {
         simulateKeyPressExt("testArea", code, code + 32, ctrlKey, altKey, false, metaKey);
      }
   }
//   123     {                                { = SHIFT + '['
   simulateKeyPressExt("testArea", 219, 91, ctrlKey, altKey, true, metaKey);
//   124     |                                | = SHIFT + '\\'
   simulateKeyPressExt("testArea", 220, 92, ctrlKey, altKey, true, metaKey);
//   125     }                                } = SHIFT + ']'
   simulateKeyPressExt("testArea", 221, 93, ctrlKey, altKey, true, metaKey);
//   126     ~                                ~ = SHIFT + '`'
   simulateKeyPressExt("testArea", 192, 96, ctrlKey, altKey, true, metaKey);
//   127     DEL                              DELETE-CHARACTER
   simulateKeyPress("testArea", 46, ctrlKey, altKey, false, metaKey);
   for (var code = 128; code <= 255; code++)
   {
      simulateKeyPress("testArea", code, ctrlKey, altKey, false, metaKey);
   }

}

/**
 * exception: plus SHIFT-TAB
 */
function arrowsInsDelHomEndPageUpDownGroup(ctrlKey, altKey, shiftKey, metaKey)
{
// 501     CURSOR-UP                        CURSOR-UP
   simulateKeyPress("testArea", 38, ctrlKey, altKey, shiftKey, metaKey);
//   502     CURSOR-DOWN                      CURSOR-DOWN
   simulateKeyPress("testArea", 40, ctrlKey, altKey, shiftKey, metaKey);
//   503     CURSOR-RIGHT                     CURSOR-RIGHT
   simulateKeyPress("testArea", 39, ctrlKey, altKey, shiftKey, metaKey);
//   504     CURSOR-LEFT                      CURSOR-LEFT
   simulateKeyPress("testArea", 37, ctrlKey, altKey, shiftKey, metaKey);
//   505     HOME                             HOME
   simulateKeyPress("testArea", 36, ctrlKey, altKey, shiftKey, metaKey);
//   506     END                              END
   simulateKeyPress("testArea", 35, ctrlKey, altKey, shiftKey, metaKey);
//   507     PAGE-UP                          PAGE-UP
   simulateKeyPress("testArea", 33, ctrlKey, altKey, shiftKey, metaKey);
//   508     PAGE-DOWN                        PAGE-DOWN
   simulateKeyPress("testArea", 34, ctrlKey, altKey, shiftKey, metaKey);
   if (!ctrlKey && !altKey && !shiftKey && !metaKey)
   {
   //   509     SHIFT-TAB                        BACK-TAB
      simulateKeyPress("testArea", 9, ctrlKey, altKey, true, metaKey);
   }
//   510     INS                              INSERT-MODE
   simulateKeyPress("testArea", 45, ctrlKey, altKey, shiftKey, metaKey);
}

function run4GLCTRLKeyTest(isGui, altKey, shiftKey, metaKey)
{
   p2j.test_helper.init(isGui);
   ctrlKeysGroup(altKey, shiftKey, metaKey);
   var startRange = 0;
   if (altKey)
   {
      startRange = 1024;
   }
   p2j.test_helper.addRange(startRange, startRange + 7);
   p2j.test_helper.addRange(startRange + 10, startRange + 12);
   p2j.test_helper.addRange(startRange + 14, startRange + 26);
   if (!altKey)
   {
      p2j.test_helper.addRange(startRange + 27, startRange + 27);
   }
   p2j.test_helper.addRange(startRange + 28, startRange + 31);
   p2j.test_helper.isPassed();
}

function testSimulator(isGui, ctrlKey, altKey, shiftKey, metaKey)
{
   p2j.test_helper.init(isGui);
// 541     CTRL-BREAK                       STOP
//   simulateKeyPress("testArea", 19, true, false, false, false);
   simulateKeyPress("testArea", 52, false, false, true, false);
   //simulateKeyPress("testArea", 'I'.codePointAt(0), true, true, true, false);

}

function run4GLKeyTest(isGui)
{
   p2j.test_helper.init(isGui);
   ctrlKeysGroup(false, false, false);
   printedKeysGroup(false, false, false);
   p2j.test_helper.addRange(0, 7);
   p2j.test_helper.addRange(10, 12);
   p2j.test_helper.addRange(14, 255);
   p2j.test_helper.addRepeatsAt(27, 2); // repeats count for ESC = 2
//   301     F1                               HELP
//   302     F2                               GO
//   303     F3
//   304     F4
//   305     F5
//   306     F6                               NEXT-FRAME
//   307     F7
//   308     F8
//   309     F9
//   310     F10
//   311     F11
//   312     F12
   for (var code = 112; code <= 123; code++)
   {
      simulateKeyPress("testArea", code, false, false, false, false);
   }
   p2j.test_helper.addRange(301, 312);
//   500
//   501     CURSOR-UP                        CURSOR-UP
//   simulateKeyPress("testArea", 38, false, false, false, false);
//   502     CURSOR-DOWN                      CURSOR-DOWN
//   simulateKeyPress("testArea", 40, false, false, false, false);
//   503     CURSOR-RIGHT                     CURSOR-RIGHT
//   simulateKeyPress("testArea", 39, false, false, false, false);
//   504     CURSOR-LEFT                      CURSOR-LEFT
//   simulateKeyPress("testArea", 37, false, false, false, false);
//   505     HOME                             HOME
//   simulateKeyPress("testArea", 36, false, false, false, false);
//   506     END                              END
//   simulateKeyPress("testArea", 35, false, false, false, false);
//   507     PAGE-UP                          PAGE-UP
//   simulateKeyPress("testArea", 33, false, false, false, false);
//   508     PAGE-DOWN                        PAGE-DOWN
//   simulateKeyPress("testArea", 34, false, false, false, false);
//   509     SHIFT-TAB                        BACK-TAB
//   simulateKeyPress("testArea", 9, false, false, true, false);
//   510     INS                              INSERT-MODE
//   simulateKeyPress("testArea", 45, false, false, false, false);
   arrowsInsDelHomEndPageUpDownGroup(false, false, false, false);
   p2j.test_helper.addRange(501, 510);

//   511     HELP-KEY
//   512     DELETE
//   513     EXECUTE
//   514     PAGE
//   515     FIND
//   516     INS-LINE
//   517     DEL-LINE
//   518     LINE-ERASE
//   519     PAGE-ERASE
//   520     SHIFT-BACKSPACE
   simulateKeyPress("testArea", 8, false, false, true, false);
   p2j.test_helper.addRange(520, 520);
//   521     SHIFT-TAB
//   522     EXIT
//   523
//   524
//   525     SHIFT-ENTER
   simulateKeyPress("testArea", 13, false, false, true, false);
   p2j.test_helper.addRange(525, 525);
//   526
//   527
//   528
//   529
//   530
//   531
//   532
//   533
//   534
//   535     ERASE
//   536     WHITE
//   537     BLUE
//   538     RED
//   539     SHIFT-ESC
   simulateKeyPress("testArea", 27, false, false, true, false);
   p2j.test_helper.addRange(539, 539);

//   540
//   541     CTRL-BREAK                       STOP
   simulateKeyPress("testArea", 19, true, false, false, false);
   p2j.test_helper.addRange(541, 541);

//   542
//   543
//   544
//   545
//   546
//   547
//   548
//   549
//   550
//   551     U1
//   552     U2
//   553     U3
//   554     U4
//   555     U5
//   556     U6
//   557     U7
//   558     U8
//   559     U9
//   560     U10
//   561
//   562
//   563
//   564
//   565
//   566
//   567
//   568
//   569
//   570
//   571
//   572
//   573
//   574
//   575
//   576
//   577
//   578
//   579
//   580
//   581
//   582
//   583
//   584
//   585
//   586     DO
//   587     SELECT
//   588     REMOVE
//   589     INSERT-HERE
//   590
//   591
//   592
//   593
//   594
//   595
//   596
//   597
//   598
//   599
//   600
//   601
//   602
//   603
//   604
//   605
//   606
//   607
//   608
//   609     MOUSE-SELECT-UP
//   610     MIDDLE-MOUSE-UP
//   611     MOUSE-MENU-UP
//   612     MOUSE-EXTEND-UP
//   613
//   614
//   615
//   616
//   617     MOUSE-SELECT-DOWN
//   618     MIDDLE-MOUSE-DOWN
//   619     MOUSE-MENU-DOWN
//   620     MOUSE-EXTEND-DOWN
//   621
//   622
//   623
//   624
//   625     MOUSE-SELECT-CLICK
//   626     MIDDLE-MOUSE-CLICK
//   627     MOUSE-MENU-CLICK
//   628     MOUSE-EXTEND-CLICK
//   629
//   630
//   631
//   632
//   633     MOUSE-SELECT-DRAG
//   634
//   635     MOUSE-MENU-DRAG
//   636     MOUSE-EXTEND-DRAG
//   637
//   638
//   639
//   640     MOUSE-MOVE
//   641
//   642
//   643
//   644
//   645
//   646
//   647
//   648
//   649     MOUSE-SELECT-DBLCLICK
//   650     MIDDLE-MOUSE-DBLCLICK
//   651     MOUSE-MENU-DBLCLICK
//   652     MOUSE-EXTEND-DBLCLICK
//   653
//   654
//   655
   //656 ... 767 SHIFT + code 144 ... 255
   for (var code = 144; code <= 255; code++)
   {
      simulateKeyPress("testArea", code, false, false, true, false);
   }
   p2j.test_helper.addRange(656, 767);
   
   //813 ...824     SHIFT + F1 ...F12
   for (var code = 112; code <= 123; code++)
   {
      simulateKeyPress("testArea", code, false, false, true, false);
   }
   p2j.test_helper.addRange(813, 824);
   //912 ...1011    SHIFT +PF1...PF99
   
   //   1013     SHIFT-CURSOR-UP
   //   1014     SHIFT-CURSOR-DOWN
   //   1015     SHIFT-CURSOR-RIGHT
   //   1016     SHIFT-CURSOR-LEFT
   
   //   1017     SHIFT-HOME
   //   1018     SHIFT-END
   //   1019     SHIFT-PAGE-UP
   //   1020     SHIFT-PAGE-DOWN
   //   1021     SHIFT-SHIFT-TAB
   //   1022     SHIFT-INS
   arrowsInsDelHomEndPageUpDownGroup(false, false, true, false);
   p2j.test_helper.addRange(1013, 1020);
   p2j.test_helper.addRange(1022, 1022);

//   1023     SHIFT-HELP-KEY

//   1024     ALT
//   1025     ALT-CTRL-A
//   1026     ALT-CTRL-B
//   1027     ALT-CTRL-C
//   1028     ALT-CTRL-D
//   1029     ALT-CTRL-E
//   1030     ALT-CTRL-F
//   1031     ALT-CTRL-G
//   1032     ALT-BACKSPACE
//   1033     ALT-CTRL-I
   simulateKeyPress("testArea", 'I'.codePointAt(0), true, true, false, false);
   p2j.test_helper.addRange(1033, 1033);
//   1034     ALT-CTRL-J
//   1035     ALT-CTRL-K
//   1036     ALT-CTRL-L
//   1037     ALT-ENTER
//   1038     ALT-CTRL-N
//   1039     ALT-CTRL-O
//   1040     ALT-CTRL-P
//   1041     ALT-CTRL-Q
//   1042     ALT-CTRL-R
//   1043     ALT-CTRL-S
//   1044     ALT-CTRL-T
//   1045     ALT-CTRL-U
//   1046     ALT-CTRL-V
//   1047     ALT-CTRL-W
//   1048     ALT-CTRL-X
//   1049     ALT-CTRL-Y

  //   1050     ALT-CTRL-Z
  //   1051     ALT-ESC
  //   1052     ALT-CTRL-\
  //   1053     ALT-CTRL-]
  //   1054     ALT-CTRL-^
  //   1055     ALT-CTRL-_
 
  //printedKeysGroup(ctrlKey, altKey, metaKey) 
  //   1056     ALT-
  //   1057     ALT-!
  //   1058     ALT-"
  //   1059     ALT-#
  //   1060     ALT-$
  //   1061     ALT-%
  //   1062     ALT-&
  //   1063     ALT-'
  //   1064     ALT-(
  //   1065     ALT-)
  //   1066     ALT-*
  //   1067     ALT-+
  //   1068     ALT-,
  //   1069     ALT--
  //   1070     ALT-.
  //   1071     ALT-/
  //   1072     ALT-0
  //   1073     ALT-1
  //   1074     ALT-2
  //   1075     ALT-3
  //   1076     ALT-4
  //   1077     ALT-5
  //   1078     ALT-6
  //   1079     ALT-7
  //   1080     ALT-8
  //   1081     ALT-9
  //   1082     ALT-:
  //   1083     ALT-;
  //   1084     ALT-<
  //   1085     ALT-=
  //   1086     ALT->
  //   1087     ALT-?
  //   1088     ALT-@
  //   1089     ALT-A
  //   1090     ALT-B
  //   1091     ALT-C
  //   1092     ALT-D
  //   1093     ALT-E
  //   1094     ALT-F
  //   1095     ALT-G
  //   1096     ALT-H
  //   1097     ALT-I
  //   1098     ALT-J
  //   1099     ALT-K
  //   1100     ALT-L
  //   1101     ALT-M
  //   1102     ALT-N
  //   1103     ALT-O
  //   1104     ALT-P
  //   1105     ALT-Q
  //   1106     ALT-R
  //   1107     ALT-S
  //   1108     ALT-T
  //   1109     ALT-U
  //   1110     ALT-V
  //   1111     ALT-W
  //   1112     ALT-X
  //   1113     ALT-Y
  //   1114     ALT-Z
  //   1115     ALT-[
  //   1116     ALT-\
  //   1117     ALT-]
  //   1118     ALT-^
  //   1119     ALT-_
  //   1120     ALT-`
//removed from this test
  //   1121     ALT-a
  //   1122     ALT-b
  //   1123     ALT-c
  //   1124     ALT-d
  //   1125     ALT-e
  //   1126     ALT-f
  //   1127     ALT-g
  //   1128     ALT-h
  //   1129     ALT-i
  //   1130     ALT-j
  //   1131     ALT-k
  //   1132     ALT-l
  //   1133     ALT-m
  //   1134     ALT-n
  //   1135     ALT-o
  //   1136     ALT-p
  //   1137     ALT-q
  //   1138     ALT-r
  //   1139     ALT-s
  //   1140     ALT-t
  //   1141     ALT-u
  //   1142     ALT-v
  //   1143     ALT-w
  //   1144     ALT-x
  //   1145     ALT-y
  //   1146     ALT-z

   //   1147     ALT-{
  //   1148     ALT-|
  //   1149     ALT-}
  //   1150     ALT-~
  //   1151     ALT-DEL
//   1152 ... 1279 ALT + code
   printedKeysGroup(false, true, false);
   p2j.test_helper.addRange(1056, 1120);
   p2j.test_helper.addRange(1147, 1279);
//   1324 ... 1423 ALT + F1 ... F99

//   1424 ... 1523 ALT + PF1 ... PF99
//   1525     ALT-CURSOR-UP
//   1526     ALT-CURSOR-DOWN
//   1527     ALT-CURSOR-RIGHT
//   1528     ALT-CURSOR-LEFT
//   1529     ALT-HOME
//   1530     ALT-END
//   1531     ALT-PAGE-UP
//   1532     ALT-PAGE-DOWN
//   1533     ALT-SHIFT-TAB
//   1534     ALT-INS
//   1535     ALT-HELP-KEY
//   1536     ALT-DELETE
//   1537     ALT-EXECUTE
//   1538     ALT-PAGE
//   1539     ALT-FIND
//   1540     ALT-INS-LINE
//   1541     ALT-DEL-LINE
//   1542     ALT-LINE-ERASE
//   1543     ALT-PAGE-ERASE
//   1544     SHIFT-ALT-BACKSPACE

//   1545     SHIFT-ALT-CTRL-I

//   1546     ALT-EXIT
//   1547
//   1548
//   1549     SHIFT-ALT-ENTER
   p2j.test_helper.isPassed();
}
